
;===============================================================
;	.proc init_system
;	cld
;	sei
;wait_vblank
;	lda vcount
;	bne wait_vblank
;	sta nmien
;	sta irqen
;	tax
;reset_all
;	sta antic,x
;	sta pokey,x
;	sta gtia,x
;	inx
;	bne reset_all
;
;	lda #$fe
;	sta portb
;	rts
;	.endp

;===============================================================

	.macro clear_memory
	.if (:1 & $ff == 0) && (:2 & $ff == 0)
	lda #>:1
	ldx #>:2
	jsr clear.clear_page_ax
	.else
	.error "Clear page not even ", :1, " ", :2
	.endif
	.endm

	.local clear
clear_page_ax
	sta clear_page_ax_adr+2
	lda #0
	tay
clear_page_ax_adr
	sta $ff00,y
	iny
	bne clear_page_ax_adr
	inc clear_page_ax_adr+2
	dex
	bne clear_page_ax_adr
	rts
	.endl

;===============================================================

	.macro copy_memory
	mwa #:1 copy.source_adr
	mwa #:2 copy.destination_adr
	mva #<:3 copy.bytes
	mva #>:3 copy.pages
	jsr copy.copy_memory
	.endm

	.local copy
copy_memory
	ldx #0
	stx bytes_count
	ldy #$00
pages = *-1
	beq no_pages
	jsr copy_loop
no_pages
	lda #$00
bytes = *-1
	beq no_bytes
	sta bytes_count
	ldy #1
copy_loop
	lda $ffff,x
source_adr = *-2
	sta $ffff,x
destination_adr = *-2
	inx
	cpx #$00
bytes_count = *-1
	bne copy_loop
	inc source_adr+1
	inc destination_adr+1
	dey
	bne copy_loop
no_bytes
	rts
	.endl

;===============================================================

	.proc sync.vcount
sync1	lda :vcount
	beq sync1
sync2	lda :vcount
	bne sync2 
	rts
	.endp

	.proc sync.cnt
	lda :cnt
loop	cmp :cnt
	beq loop
	rts
	.endp

;===============================================================
	.proc wait

	.var count	.byte

	.proc cnt
	sta count
wait1	jsr sync.cnt
	dec count
	bne wait1
	rts
	.endp

	.proc vcount
	sta count
wait1	jsr sync.vcount
	dec count
	bne wait1
	rts
	.endp

	.endp

;===============================================================
; Interrupts

	.proc init_vbi
	.if .def use_nmi
	lda #$40
	sta nmien
	rts
	.endif
	.endp

	.proc set_vbiv
	.if .def use_nmi
	jsr sync.vcount
	stx nmi.vbiv
	sty nmi.vbiv+1
	rts
	.endif
	.endp

	.proc set_dliv
	.if .def use_nmi
	jsr sync.cnt
	stx nmi.dliv
	sty nmi.dliv+1
	lda #$c0
	sta nmien
	rts
	.endif
	.endp

	.proc irq
	lda #0
	sta nmien
	sta dmactl
	lda #$ff
	sta portb
	lda #$e0
	sta chbase
	mwa #brkdl dlptr
	jsr sync.vcount
	jsr sync.vcount
	jam

brkdl	.byte $42
	.word brksm
	.byte $41
	.word brkdl
brksm	.byte 0,0,0,0
	.endp

;===============================================================
;	General purpose code generation routines

	.proc poke

	.proc poke_rts
	lda #$60
	bne poke_a
	.endp

	.proc poke_nops
	cpx #0
	beq poke_nonops
poke_nops_loop
	lda #$ea
	jsr poke_a
	dex
	bne poke_nops_loop
poke_nonops
	rts
	.endp

	.proc poke_bit_zp
	ldx #$24	;bit $00, 3 cycles
	ldy #$00
	beq poke_xy
	.endp

	.proc poke_axy
	jsr poke_a
	.endp
	
	.proc poke_xy
	txa
	jsr poke_a
	tya
	.endp
	
	.proc poke_a
	sta $ffff
	.endp
poke_adr = *-2
	inc poke_adr
	beq inc_high
	rts
inc_high
	inc poke_adr+1
	rts
	.endp
